export interface RemoteChunkLocation {
    url: string;
    /**
     * Whether not to add chunk's default extension by default. If your chunk has different
     * extension than `.chunk.bundle` you should set this flag to `true` and add extension to the `url`.
     */
    excludeExtension?: boolean;
}
export declare type RemoteChunkResolver = (chunkId: string) => Promise<RemoteChunkLocation>;
export interface StorageApi {
    getItem: (key: string) => Promise<string | null | undefined>;
    setItem: (key: string, value: string) => Promise<void>;
    removeItem: (key: string) => Promise<void>;
}
export interface ChunkManagerConfig {
    storage?: StorageApi;
    resolveRemoteChunk?: RemoteChunkResolver;
}
declare class ChunkManager {
    private cache?;
    private resolveRemoteChunk?;
    private storage?;
    configure(config: ChunkManagerConfig): void;
    private initCache;
    private saveCache;
    resolveChunk(chunkId: string): Promise<{
        url: string;
        fetch: boolean;
    }>;
    loadChunk(chunkId: string): Promise<void>;
    preloadChunk(chunkId: string): Promise<void>;
    invalidateChunks(chunksIds?: string[]): Promise<void>;
}
declare const ChunkManagerInstance: ChunkManager;
export { ChunkManagerInstance as ChunkManager };
export declare const Chunk: {
    fromDevServer(chunkId: string): string;
    fromFileSystem(chunkId: string): string;
    fromRemote(url: string, options?: {
        excludeExtension?: boolean;
    }): string;
};
