"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Chunk = exports.ChunkManager = void 0;

var _reactNative = require("react-native");

/* globals __DEV__, __CHUNKS__, __webpack_public_path__, __webpack_get_script_filename__ */
// @ts-ignore
const CACHE_KEY = `Repack.ChunkManager.Cache.${__DEV__ ? 'debug' : 'release'}`;

class ChunkManager {
  configure(config) {
    this.storage = config.storage;
    this.resolveRemoteChunk = config.resolveRemoteChunk;
  }

  async initCache() {
    if (!this.cache) {
      var _await$this$storage$g, _this$storage;

      const cache = JSON.parse((_await$this$storage$g = await ((_this$storage = this.storage) === null || _this$storage === void 0 ? void 0 : _this$storage.getItem(CACHE_KEY))) !== null && _await$this$storage$g !== void 0 ? _await$this$storage$g : 'null');
      this.cache = {
        urls: {},
        ...cache
      };
    }
  }

  async saveCache() {
    var _this$storage2;

    await ((_this$storage2 = this.storage) === null || _this$storage2 === void 0 ? void 0 : _this$storage2.setItem(CACHE_KEY, JSON.stringify(this.cache)));
  }

  async resolveChunk(chunkId) {
    var _CHUNKS__, _CHUNKS__$local;

    await this.initCache();
    let fetch = false;
    let url;

    if (__DEV__) {
      url = Chunk.fromDevServer(chunkId);
    } else if ((_CHUNKS__ = __CHUNKS__) !== null && _CHUNKS__ !== void 0 && (_CHUNKS__$local = _CHUNKS__['local']) !== null && _CHUNKS__$local !== void 0 && _CHUNKS__$local.includes(chunkId)) {
      url = Chunk.fromFileSystem(chunkId);
    } else {
      if (!this.resolveRemoteChunk) {
        throw new Error('No remote chunk resolver was provided. Did you forget to add `ChunkManager.configure({ resolveRemoteChunk: ... })`?');
      }

      const location = await this.resolveRemoteChunk(chunkId);
      url = Chunk.fromRemote(location.url, {
        excludeExtension: location.excludeExtension
      });
    }

    if (!this.cache.urls[chunkId] || this.cache.urls[chunkId] !== url) {
      fetch = true;
      this.cache.urls[chunkId] = url;
      await this.saveCache();
    }

    return {
      url: this.cache.urls[chunkId],
      fetch
    };
  }

  async loadChunk(chunkId) {
    try {
      const {
        url,
        fetch
      } = await this.resolveChunk(chunkId);
      await _reactNative.NativeModules.ChunkManager.loadChunk(chunkId, url, fetch);
    } catch (error) {
      console.error('ChunkManager.loadChunk invocation failed:', error.message, error.code ? `[${error.code}]` : '');
      throw error;
    }
  }

  async preloadChunk(chunkId) {
    try {
      const {
        url,
        fetch
      } = await this.resolveChunk(chunkId);
      await _reactNative.NativeModules.ChunkManager.preloadChunk(chunkId, url, fetch);
    } catch (error) {
      console.error('ChunkManager.preloadChunk invocation failed:', error.message, error.code ? `[${error.code}]` : '');
      throw error;
    }
  }

  async invalidateChunks(chunksIds = []) {
    try {
      await this.initCache();
      const ids = chunksIds !== null && chunksIds !== void 0 ? chunksIds : Object.keys(this.cache.urls);

      for (const chunkId of ids) {
        var _this$cache;

        (_this$cache = this.cache) === null || _this$cache === void 0 ? true : delete _this$cache.urls[chunkId];
      }

      await this.saveCache();
      await _reactNative.NativeModules.ChunkManager.invalidateChunks(ids);
    } catch (error) {
      console.error('ChunkManager.invalidateChunks invocation failed:', error.message, error.code ? `[${error.code}]` : '');
      throw error;
    }
  }

}

const ChunkManagerInstance = new ChunkManager();
exports.ChunkManager = ChunkManagerInstance;
const Chunk = {
  fromDevServer(chunkId) {
    return `${__webpack_public_path__}${__webpack_get_script_filename__(chunkId)}`;
  },

  fromFileSystem(chunkId) {
    return __webpack_get_script_filename__(`file:///${chunkId}`);
  },

  fromRemote(url, options = {}) {
    if (options.excludeExtension) {
      return url;
    }

    return __webpack_get_script_filename__(url);
  }

};
exports.Chunk = Chunk;
//# sourceMappingURL=ChunkManager.js.map