"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.AssetsPlugin = void 0;

var _AssetResolver = require("./AssetResolver");

/**
 * Plugin for loading and processing assets (images, audio, video etc) for
 * React Native applications.
 *
 * Assets processing in React Native differs from Web, Node.js or other targets. This plugin allows
 * you to use assets in the same way as you would do when using Metro.
 *
 * @category Webpack Plugin
 */
class AssetsPlugin {
  /**
   * Constructs new `AssetsPlugin`.
   *
   * @param config Plugin configuration options.
   */
  constructor(config) {
    this.config = config;
  }
  /**
   * Apply the plugin.
   *
   * @param compiler Webpack compiler instance.
   */


  apply(compiler) {
    const assetResolver = new _AssetResolver.AssetResolver(this.config, compiler);
    compiler.options.module.rules.push({
      test: assetResolver.config.test,
      use: [{
        loader: require.resolve('./assetsLoader.cjs'),
        options: {
          platform: this.config.platform,
          bundleToFile: !this.config.devServerEnabled
        }
      }]
    });
    compiler.options.resolve.plugins = (compiler.options.resolve.plugins || []).concat(assetResolver);
  }

}

exports.AssetsPlugin = AssetsPlugin;
//# sourceMappingURL=AssetsPlugin.js.map